# Example of a dde link to an excel spreadsheet. This code shows how to use
# Excel functions to carry out calculations on a list of random numbers and
# to retrieve the results afterwords.

# Copyright(C) I.B.Findleton, 1998 All rights reserved.

load dde

package require "dde" "2.0"

# Use Excel to add up some random numbers. There are actually easier ways to
# do this!

# A small loaction generator to make the script look pretty

proc Cell { row col } {

	return [format "R%dC%d" $row $col]
	}

# Generate a cell reference for use in a formula. This is not a general purpose
# routine!

proc CellReference { row col } {

	return [format "%c%d" [expr $col + 64] $row]
	}

# Generate a function that takes a range of cell references

proc MakeFunction { name from to } {

	return [format "=%s(%s:%s)" $name $from $to]
	}

# A command to put a list of random numbers into a spreadsheet and do some calculations

proc RandomSeries { row col { length 10 } } {

# Try to connect up to Excel

	set id [dde connect Excel {[Book1]Sheet1}]
	if { $id == -1 } {
		return "Excel must be running for this script to work!"
		}

	while { $row < $length } {
		dde poke $id [Cell $row $col] [expr rand() * 100]
		incr row
		}

# 	Recalculate the spreadsheet is done by entering the formulae!

	set ResultRow [expr $length + 1]
	set RangeFrom [CellReference 1 $col]
	set RangeTo [CellReference [expr $length - 1] $col]

	dde poke $id [Cell $ResultRow $col] \
		[MakeFunction SUM $RangeFrom $RangeTo]

	dde poke $id [Cell [expr $ResultRow + 1] $col] \
		[MakeFunction AVERAGE $RangeFrom $RangeTo]

	dde poke $id [Cell [expr $ResultRow + 2] $col] \
		[MakeFunction SKEW $RangeFrom $RangeTo]

	dde poke $id [Cell [expr $ResultRow + 3] $col] \
		[MakeFunction STDEV $RangeFrom $RangeTo]

#	Now retrieve the results

	set result [format "Sum %g Average %g Skew %g Std Dev %g" \
		[dde request $id [Cell $ResultRow $col]] \
		[dde request $id [Cell [expr $ResultRow + 1] $col]] \
		[dde request $id [Cell [expr $ResultRow + 2] $col]] \
		[dde request $id [Cell [expr $ResultRow + 3] $col]] \
		]

	dde disconnect $id

	return $result
	}

RandomSeries 1 1


